@@ -75,7 +75,7 @@ class Context(Base):
     # fmt: on
 
 
-def _get_field(obj: dict, field, val_type: type[T]) -> T | None:
+def _get_field(obj: Content, field, val_type: type[T]) -> T | None:
     if not isinstance(obj, dict):
         return None
     val = obj.get(field)
@@ -94,11 +94,9 @@ def _get_text(obj: Content) -> str:
     return ""
 
 
-def _modify_text(obj: Content, callback: Callable[[str], str | None]) -> str:
+def _modify_text_base(obj: ContentBase, callback: Callable[[str], str]) -> ContentBase:
     if isinstance(obj, str):
         return callback(obj)
-    if isinstance(obj, list):
-        return [_modify_text(el, callback) for el in obj]
     if isinstance(obj, dict) and obj.get("type") == "text":
         text = _get_field(obj, "text", str) or ""
         text_upd = callback(text)
@@ -106,6 +104,13 @@ def _modify_text(obj: Content, callback: Callable[[str], str | None]) -> str:
     return obj
 
 
+def _modify_text(obj: Content, callback: Callable[[str], str]) -> Content:
+    if isinstance(obj, list):
+        return [_modify_text_base(el, callback) for el in obj]
+
+    return _modify_text_base(obj, callback)
+
+
 def _get_resource_id(obj: Content) -> str | None:
     if isinstance(obj, list):
         return next((el for el in map(_get_resource_id, obj) if el), None)
@@ -122,7 +127,7 @@ def _get_resource_name(obj: Content) -> str | None:
     return None
 
 
-def _get_resource(obj: Content) -> str | None:
+def _get_resource(obj: Content) -> ResourceDict | None:
     if isinstance(obj, list):
         return next((el for el in map(_get_resource_id, obj) if el), None)
     if isinstance(obj, dict) and obj.get("type") == "resource_id":
@@ -146,9 +151,6 @@ def _get_widget(obj: Content) -> Widget | None:
     return None
 
 
-# todo fix: generalize functions _get_field
-
-
 class BaseMessage(Base):
     type: str
     content: Content = Field("", examples=["Привет"])
@@ -166,11 +168,6 @@ class BaseMessage(Base):
     def with_content(self, content: Content) -> "BaseMessage":
         return self.model_copy(update=dict(content=content))
 
-    @property
-    def body(self) -> str:
-        # legacy: eliminate after migration
-        return self.text
-
     @property
     def resource_id(self) -> str | None:
         return _get_resource_id(self.content)
@@ -208,7 +205,7 @@ class BaseMessage(Base):
         return _DT_FORMAT
 
     @staticmethod
-    def find_resource_id(msg: "BaseMessage", ext: str | None = None, type: str = None) -> str | None:
+    def find_resource_id(msg: "BaseMessage", ext: str | None = None, type: str | None = None) -> str | None:
         resource_id = msg.resource_id
         if type and type != msg.type:
             return None
@@ -218,6 +215,11 @@ class BaseMessage(Base):
             return None
         return resource_id
 
+    @staticmethod
+    def has_state(msg: "BaseMessage", state: str) -> Any | None:
+        if not msg.is_ai:
+            return None
+        return msg if msg.state == state else None
 
 class HumanMessage(BaseMessage):
     type: Literal["human"] = "human"
@@ -291,14 +293,14 @@ class Chat(Base):
     def get_last_user_message(self) -> HumanMessage | None:
         messages = self.messages
         if not messages:
-            return []
+            return None
         message = messages[-1]
         return message if isinstance(message, HumanMessage) else None
 
     def count_messages(self, func: Callable[[ChatMessage], bool] | type) -> int:
         if isinstance(func, type):
             msg_type = func
-            func = lambda msg: isinstance(msg, msg_type)
+            func = lambda msg: isinstance(msg, msg_type)  # noqa: E731
         return sum(map(func, self.messages))
 
 
